@@ -200,16 +200,13 @@ parallel.matrix <-
 parallel.data.frame <-
     function(x, data = NULL, ...)
 {
-    ocall <- sys.call(sys.parent())
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
     ccall$data <- list(x = x)
     ccall$x <- ~x
     ccall[[1]] <- quote(lattice::parallel)
-    ans <- eval.parent(ccall)
-    ans$call <- ocall
-    ans
+    eval.parent(ccall)
 }
 
 
@@ -310,7 +307,7 @@ parallel.formula <-
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent())
+    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(parallel)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
