@@ -53,14 +53,19 @@ getLabelList <- function(label, text.settings, default.label = NULL)
         ans <-
             list(label = 
                  if (is.characterOrExpression(label)) label
-                 else if (is.list(label) && (is.null(names(label)) || names(label)[1] == "")) label[[1]]
+                 else if (is.list(label) && (is.null(names(label)) ||
+                                             names(label)[1] == "")) label[[1]]
                  else default.label,
                  col = text.settings$col, cex = text.settings$cex,
                  fontfamily = text.settings$fontfamily,
                  fontface = text.settings$fontface,
                  font = text.settings$font,
                  lineheight = text.settings$lineheight)
-        if (is.list(label)) ans[names(label)] <- label
+        if (is.list(label) && !is.null(names(label)))
+        {
+            if (names(label)[1] == "") label <- label[-1]
+            ans[names(label)] <- label
+        }
     }
     else ans <- NULL
     if (is.null(ans$lab) ||
@@ -80,19 +85,54 @@ drawInViewport <-
 
 
 
-
-grobFromLabelList <- function(lab, name = "label", rot = 0)
+grobFromLabelList <- function(lab, name = "label", orient = 0)
 {
     if (is.null(lab) || (is.character(lab) && lab == "")) return (NULL)
     if (inherits(lab, "grob")) return(lab)
-    textGrob(label = lab$label, name= name, rot = rot,
-             gp =
-             gpar(col = lab$col,
-                  fontfamily = lab$fontfamily,
-                  fontface = chooseFace(lab$fontface, lab$font),
-                  lineheight = lab$lineheight,
-                  alpha = lab$alpha,
-                  cex = lab$cex))
+    process.lab <-
+        function(label, rot = orient,
+                 x = NULL, y = NULL,
+                 just = "centre",
+                 hjust = NULL, vjust = NULL,
+                 check.overlap = FALSE,
+                 font = NULL, fontfamily = NULL, fontface = NULL,
+                 ...)
+        {
+            ans <-
+                list(label = label, rot = rot, x = x, y = y,
+                     just = just, hjust = hjust, vjust = vjust,
+                     check.overlap = check.overlap)
+            ans$gplist <- 
+                gpar(fontfamily = fontfamily,
+                     fontface = chooseFace(fontface, font),
+                     ...)
+            ans
+        }
+    lab <- do.call(process.lab, lab)
+    if (is.null(lab$x))
+        lab$x <-
+            if (orient == 0) ppoints(n = length(lab$label), a = 0.5)
+            else 0.5 
+    if (is.null(lab$y))
+        lab$y <-
+            if (orient == 90) ppoints(n = length(lab$label), a = 0.5)
+            else 0.5
+    textGrob(label = lab$label,
+             x = lab$x,
+             y = lab$y,
+             name = name,
+             just = lab$just,
+             hjust = lab$hjust,
+             vjust = lab$vjust,
+             check.overlap = lab$check.overlap,
+             rot = lab$rot,
+             gp = lab$gplist)
+    ##     gpar(col = lab$col,
+    ##                   fontfamily = lab$fontfamily,
+    ##                   fontface = chooseFace(lab$fontface, lab$font),
+    ##                   lineheight = lab$lineheight,
+    ##                   alpha = lab$alpha,
+    ##                   cex = lab$cex))
 }
 
 
@@ -567,7 +607,7 @@ print.trellis <-
         grobFromLabelList(getLabelList(x$ylab,
                                        trellis.par.get("par.ylab.text"),
                                        x$ylab.default),
-                          name = trellis.grobname("ylab"), rot = 90)
+                          name = trellis.grobname("ylab"), orient = 90)
 
 
     ## get par.strip.text
@@ -1083,6 +1123,7 @@ print.trellis <-
                                       shingle.intervals = if (is.list(x$condlevels[[x$perm.cond[i]]]))
                                       do.call("rbind", x$condlevels[[x$perm.cond[i]]]) else NULL,
 
+                                      horizontal = TRUE,
                                       bg = strip.col.default.bg[i],
                                       fg = strip.col.default.fg[i],
                                       par.strip.text = par.strip.text)
@@ -1127,6 +1168,7 @@ print.trellis <-
                                            shingle.intervals = if (is.list(x$condlevels[[x$perm.cond[i]]]))
                                            do.call("rbind", x$condlevels[[x$perm.cond[i]]]) else NULL,
 
+                                           horizontal = FALSE,
                                            bg = strip.col.default.bg[i],
                                            fg = strip.col.default.fg[i],
                                            par.strip.text = par.strip.text)
